@@ -42,8 +42,11 @@ struct scsi_iovec;
 #define LIBISCSI_FEATURE_IOVECTOR (1)
 #define LIBISCSI_FEATURE_NOP_COUNTER (1)
 #define LIBISCSI_FEATURE_ISER (1)
+#define LIBISCSI_IMMEDIATE_SOCKET_WRITE (1)
 
 #define MAX_STRING_SIZE (255)
+/* RFC 3720 Section 3.2.6.1 */
+#define MAX_ISCSI_NAME_SIZE (223)
 
 /*
  * Syntax for normal and portal/discovery URLs.
@@ -183,6 +186,19 @@ EXTERN int
 iscsi_set_initial_r2t(struct iscsi_context *iscsi, enum iscsi_initial_r2t initial_r2t);
 
 
+enum iscsi_chap_auth {
+	ISCSI_CHAP_MD5 = 5,
+	ISCSI_CHAP_SHA_1 = 6,
+	ISCSI_CHAP_SHA_256 = 7,
+	ISCSI_CHAP_SHA3_256 = 8,
+};
+
+EXTERN enum iscsi_chap_auth
+iscsi_get_auth(struct iscsi_context *iscsi);
+
+EXTERN void
+iscsi_set_auth(struct iscsi_context *iscsi, enum iscsi_chap_auth auth);
+        
 /*
  * This function is used to parse an iSCSI URL into a iscsi_url structure.
  * iSCSI URL format :
@@ -335,6 +351,29 @@ enum iscsi_header_digest {
 EXTERN int iscsi_set_header_digest(struct iscsi_context *iscsi,
 			    enum iscsi_header_digest header_digest);
 
+/*
+ * Types of data digest we support. Default is NONE
+ */
+enum iscsi_data_digest {
+	ISCSI_DATA_DIGEST_NONE        = 0,
+	ISCSI_DATA_DIGEST_NONE_CRC32C = 1,
+	ISCSI_DATA_DIGEST_CRC32C_NONE = 2,
+	ISCSI_DATA_DIGEST_CRC32C      = 3,
+	ISCSI_DATA_DIGEST_LAST        = ISCSI_DATA_DIGEST_CRC32C
+};
+
+/*
+ * Set the desired data digest for a scsi context.
+ * Data digest can only be set/changed before the context
+ * is logged in to the target.
+ *
+ * Returns:
+ *  0: success
+ * <0: error
+ */
+EXTERN int iscsi_set_data_digest(struct iscsi_context *iscsi,
+			    enum iscsi_data_digest data_digest);
+
 /*
  * Specify the username and password to use for chap authentication
  */
@@ -519,6 +558,11 @@ EXTERN int iscsi_force_reconnect(struct iscsi_context *iscsi);
  */
 EXTERN int iscsi_force_reconnect_sync(struct iscsi_context *iscsi);
 
+/*
+ * Reset iscsi auto reconnect next_reconnect time to the current time.
+ */
+EXTERN void iscsi_reset_next_reconnect(struct iscsi_context *iscsi);
+
 /*
  * Asynchronous call to perform an ISCSI login.
  *
@@ -1661,6 +1705,18 @@ iscsi_set_fd_dup_cb(struct iscsi_context *iscsi,
 		    void (*cb)(struct iscsi_context *iscsi, void *opaque),
 		    void *opaque);
 
+/*
+ * MULTITHREADING
+ */        
+/*
+ * This function starts a separate service thread for multithreading support.
+ */
+EXTERN int iscsi_mt_service_thread_start(struct iscsi_context *iscsi);
+/*
+ * Shutdown multithreading support.
+ */
+EXTERN void iscsi_mt_service_thread_stop(struct iscsi_context *iscsi);
+        
 #ifdef __cplusplus
 }
 #endif
